@@ -27,10 +27,15 @@ try:
 except ImportError:
     import Queue as queue
 
-from . import DecodeError
+from .exceptions import DecodeError
 
 COMMANDS = ('ffmpeg', 'avconv')
 
+if sys.platform == "win32":
+    PROC_FLAGS = 0x08000000
+else:
+    PROC_FLAGS = 0
+
 
 class FFmpegError(DecodeError):
     pass
@@ -93,6 +98,24 @@ def popen_multiple(commands, command_args, *args, **kwargs):
                 raise
 
 
+def available():
+    """Detect whether the FFmpeg backend can be used on this system.
+    """
+    try:
+        proc = popen_multiple(
+            COMMANDS,
+            ['-version'],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            creationflags=PROC_FLAGS,
+        )
+    except OSError:
+        return False
+    else:
+        proc.wait()
+        return proc.returncode == 0
+
+
 # For Windows error switch management, we need a lock to keep the mode
 # adjustment atomic.
 windows_error_mode_lock = threading.Lock()
@@ -125,6 +148,7 @@ class FFmpegAudioFile(object):
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE,
                 stdin=self.devnull,
+                creationflags=PROC_FLAGS,
             )
 
         except OSError:
@@ -258,11 +282,34 @@ class FFmpegAudioFile(object):
 
     def close(self):
         """Close the ffmpeg process used to perform the decoding."""
-        # Kill the process if it is still running.
-        if hasattr(self, 'proc') and self.proc.returncode is None:
-            self.proc.kill()
-            self.proc.wait()
-            self.devnull.close()
+        if hasattr(self, 'proc'):
+            # First check the process's execution status before attempting to
+            # kill it. This fixes an issue on Windows Subsystem for Linux where
+            # ffmpeg closes normally on its own, but never updates
+            # `returncode`.
+            self.proc.poll()
+
+            # Kill the process if it is still running.
+            if self.proc.returncode is None:
+                self.proc.kill()
+                self.proc.wait()
+
+            # Wait for the stream-reading threads to exit. (They need to
+            # stop reading before we can close the streams.)
+            if hasattr(self, 'stderr_reader'):
+                self.stderr_reader.join()
+            if hasattr(self, 'stdout_reader'):
+                self.stdout_reader.join()
+
+            # Close the stdout and stderr streams that were opened by Popen,
+            # which should occur regardless of if the process terminated
+            # cleanly.
+            self.proc.stdout.close()
+            self.proc.stderr.close()
+
+        # Close the handle to os.devnull, which is opened regardless of if
+        # a subprocess is successfully created.
+        self.devnull.close()
 
     def __del__(self):
         self.close()
